@@ -15,7 +15,6 @@
 import argparse
 import errno
 import os
-import pwd
 import signal
 import subprocess
 import sys
@@ -27,7 +26,7 @@ from storlets.sbus.client.exceptions import SBusClientException, \
     SBusClientSendError
 from storlets.agent.common.server import command_handler, EXIT_FAILURE, \
     CommandSuccess, CommandFailure, SBusServer
-from storlets.agent.common.utils import get_logger, DEFAULT_PY2, DEFAULT_PY3
+from storlets.agent.common.utils import get_logger, DEFAULT_PY3
 
 
 class SDaemonError(Exception):
@@ -73,14 +72,7 @@ class StorletDaemonFactory(SBusServer):
         lib_dir = "/usr/local/lib/storlets"
         java_lib_dir = os.path.join(lib_dir, "java")
 
-        jar_deps = ['logback-classic-1.1.2.jar',
-                    'logback-core-1.1.2.jar',
-                    'slf4j-api-1.7.7.jar',
-                    'json_simple-1.1.jar',
-                    'SBusJavaFacade.jar',
-                    'SCommon.jar',
-                    'SDaemon.jar',
-                    '']
+        jar_deps = ['*', '']
         jar_deps = [os.path.join(java_lib_dir, x) for x in jar_deps]
         str_dmn_clspth = ':'.join(jar_deps + [storlet_path])
         str_library_path = ':'.join([lib_dir, java_lib_dir])
@@ -104,12 +96,9 @@ class StorletDaemonFactory(SBusServer):
     def get_python_args(self, daemon_language, storlet_path, storlet_name,
                         pool_size, uds_path, log_level,
                         daemon_language_version):
-        daemon_language_version = daemon_language_version or 3
-        # TODO(takashi): Drop Py2 support
-        if int(float(daemon_language_version)) == 2:
-            daemon_language_version = DEFAULT_PY2
-        else:
-            daemon_language_version = DEFAULT_PY3
+        # TODO(takashi): We currently override all 3.x by DEFAULT_PY3, but
+        #                we might need to support multiple minor versions.
+        daemon_language_version = DEFAULT_PY3
 
         python_interpreter = '/usr/bin/python%s' % daemon_language_version
         str_daemon_main_file = '/usr/local/libexec/storlets/storlets-daemon'
@@ -124,9 +113,9 @@ class StorletDaemonFactory(SBusServer):
 
     def spawn_subprocess(self, pargs, env, storlet_name):
         """
-        Launch a JVM process for some storlet daemon
+        Launch a storlet daemon process
 
-        :param pargs: Arguments for the JVM
+        :param pargs: Arguments for the storlet daemon
         :param env: Environment value
         :param storlet_name: Name of the storlet to be executed
 
@@ -134,9 +123,16 @@ class StorletDaemonFactory(SBusServer):
                                     can not check the status of the subprocess
                                     launched
         """
+        if not pargs:
+            raise SDaemonError('Invalid process arguments')
+
+        if not (os.path.exists(pargs[0]) and os.access(pargs[0], os.X_OK)):
+            raise SDaemonError('Requested runtime is unavailable')
+
         str_pargs = ' '.join(pargs)
         self.logger.debug('Starting subprocess: pargs:{0} env:{1}'
                           .format(str_pargs, env))
+
         # TODO(takashi): We had better use contextmanager
         # TODO(takashi): Where is this closed?
         try:
@@ -195,6 +191,8 @@ class StorletDaemonFactory(SBusServer):
             try:
                 resp = client.ping()
                 if resp.status:
+                    self.logger.debug('The storlet daemon {0} is started'
+                                      .format(storlet_name))
                     return True
             except SBusClientSendError:
                 pass
@@ -319,7 +317,8 @@ class StorletDaemonFactory(SBusServer):
                           .format(storlet_name, dmn_pid))
 
         if dmn_pid is None:
-            raise SDaemonError('{0} is not found'.format(storlet_name))
+            raise SDaemonError('The storlet daemon {0} is not found'
+                               .format(storlet_name))
 
         try:
             os.kill(dmn_pid, signal.SIGKILL)
@@ -331,7 +330,7 @@ class StorletDaemonFactory(SBusServer):
             return obtained_pid, obtained_code
         except OSError:
             self.logger.exception(
-                'Error when sending kill signal to the storlet daemon %s' %
+                'Error when killing the storlet daemon %s' %
                 storlet_name)
             raise SDaemonError('Failed to send kill signal to the storlet '
                                'daemon {0}'.format(storlet_name))
@@ -344,12 +343,13 @@ class StorletDaemonFactory(SBusServer):
                         stop some of the storlet daemons
         :raises SDaemonError: when failed to kill one of the storlet daemons
         """
+        self.logger.debug('Kill all storlet daemons')
         failed = []
         for storlet_name in list(self.storlet_name_to_pid):
             try:
                 self.process_kill(storlet_name)
             except SDaemonError:
-                self.logger.exception('Failed to stop the storlet daemon {0}'
+                self.logger.exception('Failed to kill the storlet daemon {0}'
                                       .format(storlet_name))
                 if try_all:
                     failed.append(storlet_name)
@@ -357,7 +357,7 @@ class StorletDaemonFactory(SBusServer):
                     raise
         if failed:
             names = ', '.join(failed)
-            raise SDaemonError('Failed to stop some storlet daemons: {0}'
+            raise SDaemonError('Failed to kill some storlet daemons: {0}'
                                .format(names))
 
     def shutdown_all_processes(self, try_all=True):
@@ -369,6 +369,7 @@ class StorletDaemonFactory(SBusServer):
         :returns: a list of the terminated storlet daemons
         :raises SDaemonError: when failed to kill one of the storlet daemons
         """
+        self.logger.debug('Shutdown all storlet daemons')
         terminated = []
         failed = []
         for storlet_name in list(self.storlet_name_to_pid):
@@ -402,9 +403,11 @@ class StorletDaemonFactory(SBusServer):
             'Shutdown the storlet daemon {0}'.format(storlet_name))
 
         dmn_pid = self.storlet_name_to_pid.get(storlet_name)
-        self.logger.debug('Storlet Daemon PID is {0}'.format(dmn_pid))
         if dmn_pid is None:
-            raise SDaemonError('{0} is not found'.format(storlet_name))
+            raise SDaemonError('PID of the storlet daemon {0} is not found'.
+                               format(storlet_name))
+        self.logger.debug('PID of the storlet daemon {0} is {1}'.
+                          format(storlet_name, dmn_pid))
 
         storlet_pipe_name = self.storlet_name_to_pipe_name[storlet_name]
         self.logger.debug('Send HALT command to {0} via {1}'.
@@ -428,6 +431,8 @@ class StorletDaemonFactory(SBusServer):
         try:
             os.waitpid(dmn_pid, 0)
             self.storlet_name_to_pid.pop(storlet_name)
+            self.logger.debug(
+                'The storlet daemon {0} is stopped'.format(storlet_name))
         except OSError:
             self.logger.exception(
                 'Error when waiting the storlet daemon {0}'.format(
@@ -448,7 +453,8 @@ class StorletDaemonFactory(SBusServer):
                         'daemon_language_version')):
                 msg = 'OK'
             else:
-                msg = '{0} is already running'.format(storlet_name)
+                msg = 'The storlet daemon {0} is already running'.format(
+                    storlet_name)
             return CommandSuccess(msg)
         except SDaemonError as err:
             self.logger.exception('Failed to start the sdaemon for {0}'
@@ -461,8 +467,8 @@ class StorletDaemonFactory(SBusServer):
         storlet_name = params['storlet_name']
         try:
             pid, code = self.process_kill(storlet_name)
-            msg = 'Storlet {0}, PID = {1}, ErrCode = {2}'.format(
-                storlet_name, pid, code)
+            msg = 'The storlet daemon {0} is stopped'.format(
+                storlet_name)
             return CommandSuccess(msg)
         except SDaemonError as err:
             self.logger.exception('Failed to kill the storlet daemon %s' %
@@ -498,15 +504,18 @@ class StorletDaemonFactory(SBusServer):
     @command_handler
     def halt(self, dtg):
         try:
-            terminated = self.shutdown_all_processes()
-            msg = '; '.join(['%s: terminated' % x for x in terminated])
+            self.shutdown_all_processes()
+            msg = 'Stopped all storlet daemons. Terminating.'
             return CommandSuccess(msg, False)
         except SDaemonError as err:
-            self.logger.exception('Failed to halt some storlet daemons')
+            self.logger.exception('Failed to stop some storlet daemons')
             return CommandFailure(err.args[0], False)
 
     def _terminate(self):
-        pass
+        self.shutdown_all_processes()
+
+    def _force_terminate(self):
+        self.process_kill_all()
 
 
 def main():
@@ -526,12 +535,6 @@ def main():
 
     try:
         SBus.start_logger("DEBUG", container_id=opts.container_id)
-
-        # Impersonate the swift user
-        pw = pwd.getpwnam('swift')
-        os.setresgid(pw.pw_gid, pw.pw_gid, pw.pw_gid)
-        os.setresuid(pw.pw_uid, pw.pw_uid, pw.pw_uid)
-
         # create an instance of daemon_factory
         factory = StorletDaemonFactory(opts.sbus_path, logger,
                                        opts.container_id)
