@@ -14,8 +14,8 @@
 # limitations under the License.
 from contextlib import contextmanager
 import errno
-import mock
 import unittest
+from unittest import mock
 
 from storlets.sbus import command as sbus_cmd
 from storlets.sbus.client import SBusResponse
@@ -23,7 +23,7 @@ from storlets.sbus.client.exceptions import SBusClientSendError
 
 from storlets.agent.daemon_factory.server import SDaemonError, \
     StorletDaemonFactory
-from storlets.agent.common.utils import DEFAULT_PY2, DEFAULT_PY3
+from storlets.agent.common.utils import DEFAULT_PY3
 
 from tests.unit import FakeLogger
 from tests.unit.agent.common import test_server
@@ -35,9 +35,15 @@ class DummyDatagram(object):
 
 
 class TestStorletDaemonFactory(unittest.TestCase):
+    # Class paths used in mock.patch
     base_path = 'storlets.agent.daemon_factory.server'
-    kill_path = base_path + '.os.kill'
-    waitpid_path = base_path + '.os.waitpid'
+    os_environ_path = base_path + '.os.environ'
+    os_path_exists_path = base_path + '.os.path.exists'
+    os_access_path = base_path + '.os.access'
+    os_kill_path = base_path + '.os.kill'
+    os_waitpid_path = base_path + '.os.waitpid'
+    subprocess_popen_path = base_path + '.subprocess.Popen'
+    time_sleep_path = base_path + '.time.sleep'
 
     @contextmanager
     def _mock_sbus_client(self, method):
@@ -55,8 +61,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
     def test_get_jvm_args(self):
         dummy_env = {'CLASSPATH': '/default/classpath',
                      'LD_LIBRARY_PATH': '/default/ld/library/path'}
-        with mock.patch('storlets.agent.daemon_factory.server.os.environ',
-                        dummy_env):
+        with mock.patch(self.os_environ_path, dummy_env):
             pargs, env = self.dfactory.get_jvm_args(
                 'java', 'path/to/storlet/a', 'Storlet-1.0.jar',
                 1, 'path/to/uds/a', 'DEBUG')
@@ -69,13 +74,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
             self.assertIn('CLASSPATH', env)
             self.assertEqual(
                 ['/default/classpath',
-                 '/usr/local/lib/storlets/java/logback-classic-1.1.2.jar',
-                 '/usr/local/lib/storlets/java/logback-core-1.1.2.jar',
-                 '/usr/local/lib/storlets/java/slf4j-api-1.7.7.jar',
-                 '/usr/local/lib/storlets/java/json_simple-1.1.jar',
-                 '/usr/local/lib/storlets/java/SBusJavaFacade.jar',
-                 '/usr/local/lib/storlets/java/SCommon.jar',
-                 '/usr/local/lib/storlets/java/SDaemon.jar',
+                 '/usr/local/lib/storlets/java/*',
                  '/usr/local/lib/storlets/java/',
                  'path/to/storlet/a'],
                 env['CLASSPATH'].split(':'))
@@ -89,15 +88,12 @@ class TestStorletDaemonFactory(unittest.TestCase):
 
     def test_get_python_args(self):
         self._test_get_python_args(None, DEFAULT_PY3)
-        self._test_get_python_args(DEFAULT_PY2, DEFAULT_PY2)
-        self._test_get_python_args(2, DEFAULT_PY2)
         self._test_get_python_args(DEFAULT_PY3, DEFAULT_PY3)
         self._test_get_python_args(3, DEFAULT_PY3)
 
     def _test_get_python_args(self, version, expected):
         dummy_env = {'PYTHONPATH': '/default/pythonpath'}
-        with mock.patch('storlets.agent.daemon_factory.server.os.environ',
-                        dummy_env):
+        with mock.patch(self.os_environ_path, dummy_env):
             pargs, env = self.dfactory.get_python_args(
                 'python', 'path/to/storlet', 'test_storlet.TestStorlet',
                 1, 'path/to/uds', 'DEBUG', version)
@@ -121,10 +117,14 @@ class TestStorletDaemonFactory(unittest.TestCase):
                 self.pid = pid
                 self.stderr = mock.MagicMock()
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen, \
-                mock.patch(self.base_path + '.time.sleep'), \
-                mock.patch(self.waitpid_path) as waitpid, \
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen, \
+                mock.patch(self.time_sleep_path), \
+                mock.patch(self.os_waitpid_path) as waitpid, \
                 self._mock_sbus_client('ping') as ping:
+            exists.return_value = True
+            access.return_velue = True
             popen.side_effect = [FakePopenObject(1000),
                                  FakePopenObject(1001)]
             waitpid.return_value = 0, 0
@@ -132,14 +132,19 @@ class TestStorletDaemonFactory(unittest.TestCase):
             self.dfactory.spawn_subprocess(
                 ['arg0', 'argv1', 'argv2'],
                 {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
             self.assertEqual((1000, 1), waitpid.call_args[0])
             self.assertEqual({'storleta': 1000},
                              self.dfactory.storlet_name_to_pid)
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen, \
-                mock.patch(self.base_path + '.time.sleep'), \
-                mock.patch(self.waitpid_path) as waitpid, \
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen, \
+                mock.patch(self.time_sleep_path), \
+                mock.patch(self.os_waitpid_path) as waitpid, \
                 self._mock_sbus_client('ping') as ping:
+            exists.return_value = True
+            access.return_velue = True
             popen.side_effect = [FakePopenObject(1000),
                                  FakePopenObject(1001)]
             waitpid.return_value = 0, 0
@@ -148,13 +153,18 @@ class TestStorletDaemonFactory(unittest.TestCase):
                 self.dfactory.spawn_subprocess(
                     ['arg0', 'argv1', 'argv2'],
                     {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
             self.assertEqual((1000, 1), waitpid.call_args[0])
             self.assertEqual({'storleta': 1000},
                              self.dfactory.storlet_name_to_pid)
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen, \
-                mock.patch(self.base_path + '.time.sleep'), \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen, \
+                mock.patch(self.time_sleep_path), \
+                mock.patch(self.os_waitpid_path) as waitpid:
+            exists.return_value = True
+            access.return_velue = True
             popen.side_effect = [FakePopenObject(1000),
                                  FakePopenObject(1001)]
             waitpid.return_value = 1000, -1
@@ -162,28 +172,61 @@ class TestStorletDaemonFactory(unittest.TestCase):
                 self.dfactory.spawn_subprocess(
                     ['arg0', 'argv1', 'argv2'],
                     {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
             self.assertEqual((1000, 1), waitpid.call_args[0])
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen:
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.subprocess_popen_path) as popen:
+            exists.return_value = False
+            with self.assertRaises(SDaemonError):
+                self.dfactory.spawn_subprocess(
+                    ['arg0', 'argv1', 'argv2'],
+                    {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
+            popen.assert_not_called()
+
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen:
+            exists.return_value = True
+            access.return_value = False
+            with self.assertRaises(SDaemonError):
+                self.dfactory.spawn_subprocess(
+                    ['arg0', 'argv1', 'argv2'],
+                    {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
+            popen.assert_not_called()
+
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen:
+            exists.return_value = True
+            access.return_value = False
             popen.side_effect = OSError()
             with self.assertRaises(SDaemonError):
                 self.dfactory.spawn_subprocess(
                     ['arg0', 'argv1', 'argv2'],
                     {'envk0': 'envv0'}, 'storleta')
+            self.assertEqual(('arg0',), exists.call_args[0])
+
+        with mock.patch(self.os_path_exists_path) as exists:
+            with self.assertRaises(SDaemonError):
+                self.dfactory.spawn_subprocess([], {}, 'storleta')
+            self.assertEqual(0, exists.call_count)
 
     def test_wait_for_daemon_to_initialize(self):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a'}
 
         with self._mock_sbus_client('ping') as ping, \
-                mock.patch(self.base_path + '.time.sleep'):
+                mock.patch(self.time_sleep_path):
             ping.return_value = SBusResponse(True, 'OK')
             self.assertTrue(
                 self.dfactory.wait_for_daemon_to_initialize('storleta'))
             self.assertEqual(1, ping.call_count)
 
         with self._mock_sbus_client('ping') as ping, \
-                mock.patch(self.base_path + '.time.sleep'):
+                mock.patch(self.time_sleep_path):
             ping.return_value = SBusResponse(False, 'NG')
             self.assertFalse(
                 self.dfactory.wait_for_daemon_to_initialize('storleta'))
@@ -194,7 +237,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('ping') as ping, \
-                mock.patch(self.base_path + '.time.sleep'):
+                mock.patch(self.time_sleep_path):
             ping.side_effect = SBusClientSendError()
             self.assertFalse(
                 self.dfactory.wait_for_daemon_to_initialize('storleta'))
@@ -212,10 +255,14 @@ class TestStorletDaemonFactory(unittest.TestCase):
                 self.pid = pid
                 self.stderr = mock.MagicMock()
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen, \
-                mock.patch(self.base_path + '.time.sleep'), \
-                mock.patch(self.waitpid_path) as waitpid, \
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen, \
+                mock.patch(self.time_sleep_path), \
+                mock.patch(self.os_waitpid_path) as waitpid, \
                 self._mock_sbus_client('ping') as ping:
+            exists.return_value = True
+            access.return_value = True
             popen.side_effect = [FakePopenObject(1000),
                                  FakePopenObject(1001)]
             waitpid.return_value = 0, 0
@@ -229,7 +276,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Already running
         self.dfactory.storlet_name_to_pid = {'storleta': 1000}
         self.dfactory.storlet_name_to_pipe_name = {'storleta': 'path/to/uds/a'}
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 0, 0
             self.assertFalse(self.dfactory.process_start_daemon(
                 'java', 'path/to/storlet/a', 'storleta', 1, 'path/to/uds/a',
@@ -245,7 +292,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 0, 0
             self.assertTrue(
                 self.dfactory.get_process_status_by_name('storleta'))
@@ -256,40 +303,40 @@ class TestStorletDaemonFactory(unittest.TestCase):
             self.dfactory.get_process_status_by_name('storletc'))
 
     def test_get_process_status_by_pid(self):
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 0, 0
             self.assertTrue(
                 self.dfactory.get_process_status_by_pid(1000, 'storleta'))
             self.assertEqual(1, waitpid.call_count)
             self.assertEqual((1000, 1), waitpid.call_args[0])
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 1000, 0
             self.assertFalse(
                 self.dfactory.get_process_status_by_pid(1000, 'storleta'))
             self.assertEqual(1, waitpid.call_count)
             self.assertEqual((1000, 1), waitpid.call_args[0])
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = OSError(errno.ESRCH, '')
             self.assertFalse(
                 self.dfactory.get_process_status_by_pid(1000, 'storleta'))
             self.assertEqual(1, waitpid.call_count)
             self.assertEqual((1000, 1), waitpid.call_args[0])
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = OSError(errno.EPERM, '')
             exc_pattern = ('^No permission to access the storlet daemon'
                            ' storleta$')
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern):
+            with self.assertRaisesRegex(SDaemonError, exc_pattern):
                 self.dfactory.get_process_status_by_pid(1000, 'storleta')
             self.assertEqual(1, waitpid.call_count)
             self.assertEqual((1000, 1), waitpid.call_args[0])
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = OSError()
             exc_pattern = '^Unknown error$'
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern):
+            with self.assertRaisesRegex(SDaemonError, exc_pattern):
                 self.dfactory.get_process_status_by_pid(1000, 'storleta')
             self.assertEqual(1, waitpid.call_count)
             self.assertEqual((1000, 1), waitpid.call_args[0])
@@ -298,8 +345,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Success
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 1000, 0
             self.assertEqual((1000, 0),
                              self.dfactory.process_kill('storleta'))
@@ -311,8 +358,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # When failed to send kill to the storlet daemon
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             kill.side_effect = OSError()
             with self.assertRaises(SDaemonError):
                 self.dfactory.process_kill('storleta')
@@ -324,8 +371,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # When failed to wait
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = OSError()
             with self.assertRaises(SDaemonError):
                 self.dfactory.process_kill('storleta')
@@ -337,8 +384,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # if the storlet daemon is not recognised
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             with self.assertRaises(SDaemonError):
                 self.dfactory.process_kill('storletc')
             self.assertEqual(0, kill.call_count)
@@ -350,8 +397,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Success
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = [(1000, 0), (1001, 0)]
             self.dfactory.process_kill_all()
             self.assertEqual(2, kill.call_count)
@@ -360,8 +407,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
 
         # Success (no processes)
         self.dfactory.storlet_name_to_pid = {}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             self.dfactory.process_kill_all()
             self.assertEqual(0, kill.call_count)
             self.assertEqual(0, waitpid.call_count)
@@ -370,11 +417,11 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Failure (try_all = True)
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             kill.side_effect = OSError()
             exc_pattern = '^Failed to kill some storlet daemons: .*'
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern) as e:
+            with self.assertRaisesRegex(SDaemonError, exc_pattern) as e:
                 self.dfactory.process_kill_all()
             self.assertIn('storleta', str(e.exception))
             self.assertIn('storletb', str(e.exception))
@@ -386,12 +433,12 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Failure (try_all = False)
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path) as waitpid:
             kill.side_effect = OSError()
             exc_pattern = ('^Failed to send kill signal to the storlet daemon '
                            'storlet[a-b]$')
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern):
+            with self.assertRaisesRegex(SDaemonError, exc_pattern):
                 self.dfactory.process_kill_all(False)
             self.assertEqual(1, kill.call_count)
             self.assertEqual(0, waitpid.call_count)
@@ -405,7 +452,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path):
+                mock.patch(self.os_waitpid_path):
             halt.return_value = SBusResponse(True, 'OK')
             terminated = self.dfactory.shutdown_all_processes()
             self.assertEqual(2, len(terminated))
@@ -420,10 +467,10 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'patha', 'storletb': 'pathb'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path) as waitpid:
+                mock.patch(self.os_waitpid_path) as waitpid:
             halt.side_effect = SBusClientSendError()
             exc_pattern = '^Failed to shutdown some storlet daemons: .*'
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern) as e:
+            with self.assertRaisesRegex(SDaemonError, exc_pattern) as e:
                 self.dfactory.shutdown_all_processes()
             self.assertIn('storleta', str(e.exception))
             self.assertIn('storletb', str(e.exception))
@@ -438,11 +485,11 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'patha', 'storletb': 'pathb'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path) as waitpid:
+                mock.patch(self.os_waitpid_path) as waitpid:
             halt.side_effect = SBusClientSendError()
             exc_pattern = ('^Failed to send halt command to the storlet '
                            'daemon storlet[a-b]$')
-            with self.assertRaisesRegexp(SDaemonError, exc_pattern):
+            with self.assertRaisesRegex(SDaemonError, exc_pattern):
                 self.dfactory.shutdown_all_processes(False)
             self.assertEqual(1, halt.call_count)
             self.assertEqual(0, waitpid.call_count)
@@ -456,7 +503,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path):
+                mock.patch(self.os_waitpid_path):
             halt.return_value = SBusResponse(True, 'OK')
             self.dfactory.shutdown_process('storleta')
             self.assertEqual({'storletb': 1001},
@@ -468,7 +515,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path) as waitpid:
+                mock.patch(self.os_waitpid_path) as waitpid:
             halt.side_effect = SBusClientSendError()
             with self.assertRaises(SDaemonError):
                 self.dfactory.shutdown_process('storleta')
@@ -482,7 +529,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path) as waitpid:
+                mock.patch(self.os_waitpid_path) as waitpid:
             halt.return_value = SBusResponse(True, 'OK')
             waitpid.side_effect = OSError()
             with self.assertRaises(SDaemonError):
@@ -514,11 +561,15 @@ class TestStorletDaemonFactory(unittest.TestCase):
                 self.pid = pid
                 self.stderr = mock.MagicMock()
 
-        with mock.patch(self.base_path + '.subprocess.Popen') as popen, \
-                mock.patch(self.base_path + '.time.sleep'), \
-                mock.patch(self.waitpid_path) as waitpid, \
+        with mock.patch(self.os_path_exists_path) as exists, \
+                mock.patch(self.os_access_path) as access, \
+                mock.patch(self.subprocess_popen_path) as popen, \
+                mock.patch(self.time_sleep_path), \
+                mock.patch(self.os_waitpid_path) as waitpid, \
                 self._mock_sbus_client('ping') as ping, \
                 self._mock_sbus_client('start_daemon') as start_daemon:
+            exists.return_value = True
+            access.return_value = True
             popen.side_effect = [FakePopenObject(1000),
                                  FakePopenObject(1001)]
             waitpid.return_value = 0, 0
@@ -532,7 +583,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Already running
         self.dfactory.storlet_name_to_pid = {'storleta': 1000}
         self.dfactory.storlet_name_to_pipe_name = {'storleta': 'path/to/uds/a'}
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 0, 0
             ret = self.dfactory.start_daemon(DummyDatagram(prms))
             self.assertTrue(ret.status)
@@ -551,8 +602,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Success
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000}
-        with mock.patch(self.kill_path), \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path), \
+                mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 1000, 0
             resp = self.dfactory.stop_daemon(
                 DummyDatagram({'storlet_name': 'storleta'}))
@@ -564,8 +615,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Failure
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000}
-        with mock.patch(self.kill_path) as kill, \
-                mock.patch(self.waitpid_path):
+        with mock.patch(self.os_kill_path) as kill, \
+                mock.patch(self.os_waitpid_path):
             kill.side_effect = OSError('ERROR')
             resp = self.dfactory.stop_daemon(
                 DummyDatagram({'storlet_name': 'storleta'}))
@@ -579,7 +630,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 0, 0
             resp = self.dfactory.daemon_status(
                 DummyDatagram({'storlet_name': 'storleta'}))
@@ -588,7 +639,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
                              resp.message)
             self.assertTrue(resp.iterable)
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.return_value = 1000, 0
             resp = self.dfactory.daemon_status(
                 DummyDatagram({'storlet_name': 'storleta'}))
@@ -597,7 +648,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
                              resp.message)
             self.assertTrue(resp.iterable)
 
-        with mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = OSError()
             resp = self.dfactory.daemon_status(
                 DummyDatagram({'storlet_name': 'storleta'}))
@@ -611,7 +662,7 @@ class TestStorletDaemonFactory(unittest.TestCase):
         self.dfactory.storlet_name_to_pipe_name = \
             {'storleta': 'path/to/uds/a', 'storletb': 'path/to/uds/b'}
         with self._mock_sbus_client('halt') as halt, \
-                mock.patch(self.waitpid_path):
+                mock.patch(self.os_waitpid_path):
             halt.return_value = SBusResponse(True, 'OK')
             resp = self.dfactory.halt(DummyDatagram())
             self.assertTrue(resp.status)
@@ -623,8 +674,8 @@ class TestStorletDaemonFactory(unittest.TestCase):
         # Success
         self.dfactory.storlet_name_to_pid = \
             {'storleta': 1000, 'storletb': 1001}
-        with mock.patch(self.kill_path), \
-                mock.patch(self.waitpid_path) as waitpid:
+        with mock.patch(self.os_kill_path), \
+                mock.patch(self.os_waitpid_path) as waitpid:
             waitpid.side_effect = [(1000, 0), (1001, 0)]
             resp = self.dfactory.stop_daemons(DummyDatagram())
             self.assertTrue(resp.status)
